import numpy as np
import time
import generator_functions as gen_fun, functions as fun


######
def set_up_device(scope, generator, freq, volt, directory, triger_ch='CH1', voltage_ch='CH2', current_ch='CH3'):
    # stacked display mode
    scope.write('DISplay:WAVEView1:VIEWStyle OVErlay')

    # switch on high resolution mode
    scope.write('ACQuire:MODe HIRes')

    # ustawiam ktore kanaly wlczone
    scope.write('DISplay:GLObal:CH1:STATE OFF')
    scope.write('DISplay:GLObal:CH2:STATE OFF')
    scope.write('DISplay:GLOba1:CH3:STATE OFF')
    scope.write('DISplay:GLObal:CH4:STATE OFF')
    scope.write('DISplay:GLOba1:CH5:STATE OFF')
    scope.write('DISplay:GLOba1:CH6:STATE OFF')
    time.sleep(0.01)
    scope.write('DISplay:GLOba1:' + str(triger_ch) + ':STATE ON')
    scope.write('DISplay:GLOba1:' + str(voltage_ch) + ':STATE ON')
    scope.write('DISplay:GLOba1:' + str(current_ch) + ':STATE ON')

    # ustawienie podstawy, aby zalezala od czestotliwosci, to musi byc przed kursorami!
    P = str(np.format_float_scientific(6 * (1 / (freq * 1000) / 10)))  # przez 10 bo 10 dzialek
    scope.write('HORizontal:MODE:SCAle ' + P)

    # ustawienie dla kazdego z kanalow (mA/V) /div
    scope.write(str(triger_ch) + ':SCALE 2.0E+0')
    scope.write(str(current_ch) + ':SCALE 100.0E-3')  # UWAGA TO DO OGARNIECIA!
    scope.write(str(voltage_ch) + ':SCALE ' + np.format_float_scientific(np.ceil(volt * 2 / 5), 3).__str__())

    # kursory
    scope.write('DISplay:WAVEView1:CURSor:CURSOR1:STATE ON')  # dziala, wlacza kursor
    scope.write('DISplay:WAVEView1:CURSor:CURSOR1:MODe INDEPENDENT')
    scope.write('DATA:ENCdg ASCIi')

    # triger z kanalu triger_ch ustawiony jako timeout
    scope.write('TRIGger:A:TYPe TIMEOut')
    scope.write('TRIGger:A:TIMEOut:SOUrce ' + str(triger_ch))

    # triger ustawiamy na 50% automatycznie
    # scope.write('TRIGger:A SETLevel')
    # scope.write('TRIGger:A:TYPe EDGe')

    # time.sleep(0.01)
    # print(triger_ch)
    # print(current_ch)
    # print(voltage_ch)

    # scope.write('SAVEON:WAVEFORM ON')
    scope.write('SAVEON:WAVEform:FILEFormat SPREADSheet')

    # set gating  to cursor and set it to global
    scope.write('SAVE:WAVeform:GATing CURSor')

    # ustawienie trigera na 50% wartosci ktora bedzie wyzwalany
    gen_fun.enable_output(generator)
    time.sleep(2)
    scope.write('TRIGger:A SETLEVEL')
    time.sleep(2)
    gen_fun.disable_output(generator)

    # ustawiam, zeby dane byly zapisywane na dwoch bajtach oraz aby byly sciagne z pomiedzy kursorow
    scope.write('DATa SNAp')
    scope.write('DATA:WIDTH 2')

    # tworze nowy folder na dane na pendrivie
    new_dir = '"E:/experiment/' + directory.split('\\')[-1]+'"'
    scope.write('FILESystem:MKDir ' + new_dir)
    scope.write('FILESystem:CWD ' + new_dir)
    scope.write('SAVEON:FILE:DEST ' + new_dir)
    scope.write('SAVe:SETUp:INCLUDEREFs ON')  # specifies including displayed reference waveforms in saved setups

    print('New Dir from Setup Osc', new_dir)
    print("Cuurent working directory z set_up: ", scope.query('FILESystem:CWD?'))
    print("New FILE DEST: ", scope.query('SAVEON:FILE:DEST?'))
    print("Info o tym czy zapisujemy duzo danych: ", scope.query('SAVe:SETUp:INCLUDEREFs?'))

    print("#?#?#?#?#?#?#?#?#?#?#?#?#?#?#?#?#?#?#?#")
    print("KONIEC SET UPu OSCYLOSKOPU")
    print("#?#?#?#?#?#?#?#?#?#?#?#?#?#?#?#?#?#?#?#")

    # mozna dodac, aby podsawa czasu byly sekundy a nie herce
######

######
def update_device(scope, freq, voltage, voltage_ch):
    # switch on high resolution mode
    # scope.write('ACQuire:MODe HIRes')

    # ustawiam ekran tak, aby miescil 6 okresow:
    S = str(np.format_float_scientific((6 * (1 / (freq * 10000))), 2))
    scope.write('HORizontal:MODE:SCAle ' + S)

    # ustaw skale tak, aby miescila zakres napiecia
    scope.write(str(voltage_ch) + ':SCALE ' + np.format_float_scientific(np.ceil(voltage * 2 / 5), 3))
    # print("VOLTAGE: ", voltage)
    # print("SCIENTIFIC VOLTAGE: ", str(voltage_ch)+':SCALE '+np.format_float_scientific(np.ceil(voltage*2/5), 3))

    # opoznienie trigera
    O = str(np.format_float_scientific(1 / (4 * freq * 1000), 3))
    scope.write(':TRIGger:A:TIMEOut:TIMe ' + O)

    # triger ustawiamy na 50% automatycznie !to jest juz ustawione
    # scope.write('TRIGger:A SETLevel')
    # time.sleep(2)

    # ustaw kursory na 0 oraz T + 10ns
    scope.write('DISplay:WAVEView1:CURSor:CURSOR1:VBArs:APOSition ' + str(
        np.format_float_scientific(0, 3)))
    Tplus10 = str(np.format_float_scientific((1 / (1000 * freq) + pow(10, -8)), 3))
    scope.write('DISplay:WAVEView1:CURSor:CURSOR1:VBArs:BPOSition ' + Tplus10)
######


def download_data(scope, frequency, voltage, directory, current_ch='CH3', voltage_ch='CH2'):
    new_dir = directory + '\\' + str(voltage) + 'V__' + str(frequency) + 'kHz.csv'
    print("DIRECTORY from download data: ", new_dir)

    # ostateczny update trigera przed sciagnieciem danych. Ustawienie na 50% wartosci obecnej
    # scope.write('TRIGger:A SETLevel')

    # opoznienie, moze sie okazac niepotrzebne
    # time.sleep(0.1)

    # ustawienia podstawowe oscyloskopu do poprawnego pobrania danych
    scope.write('HEADer OFF')

    # triger oraz kursory juz sa ustawione

    # CURRENT
    scope.write('DATa:SOUrce ' + str(current_ch))
    time.sleep(0.2)
    scope.write('DATa SNAp')

    current_val = scope.query_ascii_values('CURV?',
                                           converter='d',
                                           container=np.array)
    time.sleep(0.5)
    out_WFM1 = str(scope.query('WFMOUTPRE?')).split(';')

    # VOLTAGE
    scope.write('DATa:SOUrce ' + str(voltage_ch))
    time.sleep(0.2)

    scope.write('DATa SNAp')
    voltage_val = scope.query_ascii_values('CURV?',
                                           converter='d',
                                           container=np.array)
    time.sleep(0.5)
    out_WFM2 = str(scope.query('WFMOUTPRE?')).split(';')

    # print("curret ch: ", current_ch)
    # print("voltage ch: ", voltage_ch)
    # print(':DATa:SOUrce ' + str(current_ch))
    # print(':DATa:SOUrce ' + str(voltage_ch))

    # print("curr max: ", np.amax(current_val))
    # print("volt max: ", np.amax(voltage_val))
    print("WFM1: ", out_WFM1)
    print("WFM2: ", out_WFM2)

    print("Current value: ", current_val)
    print("Voltage value: ", voltage_val)

    # wyswietlam pomocniczo jeszcze info o tym, gdzie sa kursory
    # print(scope.query('DISplay:MATHFFTView<x>:CURSor:HBArs:DELTa?'))

    # normalizuje
    mul1, mul2 = fun.normalize_from_string(out_WFM1, out_WFM2)
    # print("Current value: ", current_val)
    # print("Voltage value: ", voltage_val)
    # print(out_WFM1)
    # print(out_WFM2)
    # print('\n' * 5)

    current_val = current_val * mul1
    voltage_val = voltage_val * mul2

    # Wyswietlam dane
    fun.show_data(current_val, voltage_val, frequency, voltage)

    # Zapisuje dane
    fun.save_data_to_file(new_dir, current_val, voltage_val)

    scope.write('HEADer ON')


def save_in_pendrive(scope, frequency, voltage, directory, current_ch='CH3', voltage_ch='CH2'):
    # ostateczny update trigera przed sciagnieciem danych. Ustawienie na 50% wartosci obecnej
    # scope.write('TRIGger:A SETLevel')

    # nowej nazwy pliku
    new_name = str(voltage) + 'V__' + str(frequency) + 'kHz.csv"'
    new_dir = '"E:/experiment/' + directory.split('\\')[-1]+'/'

    print("Cuurent working directory: ", scope.query('FILESystem:CWD?'))

    print("Nowa nazwa pliku: ", new_dir+new_name)



    # przeslanie instrukcji do oscyloskopu
    # scope.write('SAVEON:FILE:DEST ' + '"' + new_dir + '/"')

    # setup i dane
    scope.write('SAVEON:FILE:NAME ' + new_name)  # Sets or queries the file name to use when SAVEON:TRIGer is ON.

    #set up triger on as well as trigger saveon
    scope.write('SAVEON:WAVeform ON; SAVEON:TRIGGER ON')

    scope.write('SAVe:WAVEform ALL, "' + new_name)

    # time.sleep(5)
    # print('FILESystem:DIR?: ', scope.query('FILESystem:DIR?'))
    # print('FILESystem:CWD?: ', scope.query('FILESystem:CWD?'))
    # print('SAVEON:FILE:NAME?: ', scope.query('SAVEON:FILE:NAME?'))
    # print('FILESystem:MKDir ' + '"' + new_dir)
    # print('SAVe:SETUp:INCLUDEREFs?: ', scope.query('SAVe:SETUp:INCLUDEREFs?'))
    # print('SAVEON:FILE:DEST?: ', scope.query('SAVEON:FILE:DEST?'))
    # print('SAVEON:WAVeform?: ', scope.query('SAVEON:WAVeform?'))
    # print('SAVEON:TRIGGER?: ', scope.query('SAVEON:TRIGGER?'))
    # print('SAVe:WAVEform:SOURCELIst?: ', scope.query('SAVe:WAVEform:SOURCELIst?'))


def enable_reading(scope):
    scope.write('SAVEON:WAVeform OFF; SAVEON:TRIGGER OFF')


def finish(scope):
    scope.close()
